//
//  CUMapPathView.m
//  CUMapViewExample
//
//  Created by curer yg on 12-3-21.
//  Copyright (c) 2012年 zhubu. All rights reserved.
//
// copy from MapView  Created by kadir pekel on 2/7/10.

#import "CUMapPathView.h"
#import "ASIHTTPRequest.h"

@interface CUMapPathView()

@property (nonatomic, retain) MKMapView *mapView;
@property (nonatomic, retain) UIImageView* routeView;
@property (nonatomic, retain) NSArray *routeArray;

- (NSMutableArray *)decodePolyLine:(NSMutableString *)encoded;
- (void)updateRouteView:(NSArray *)routes;
- (void)calculateRoutesFrom:(CLLocationCoordinate2D)from to:(CLLocationCoordinate2D)to;
- (void)centerMap:(NSArray *)routes;

@end

@implementation CUMapPathView

@synthesize lineColor;
@synthesize mapView;
@synthesize routeView;
@synthesize routeArray;

- (id)initWithFrame:(CGRect)frame
{
	self = [super initWithFrame:frame];
	if (self != nil) {
		mapView = [[MKMapView alloc] initWithFrame:CGRectMake(0, 0, frame.size.width, frame.size.height)];
		mapView.showsUserLocation = YES;
		[mapView setDelegate:self];
		[self addSubview:mapView];
		routeView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, mapView.frame.size.width, mapView.frame.size.height)];
		routeView.userInteractionEnabled = NO;
		[mapView addSubview:routeView];
		
		self.lineColor = [UIColor colorWithWhite:0.2 alpha:0.5];
	}
    
	return self;
}

- (void)dealloc
{
    self.routeView = nil;
    self.lineColor = nil;
    self.mapView = nil;
    self.routeArray = nil;
    
    [super dealloc];
}

- (void)showRouteFrom:(Place *)src to:(Place *)des
{
    /*
    if(routes) {
		[mapView removeAnnotations:[mapView annotations]];
		[routes release];
	}*/
    
    [mapView removeAnnotations:[mapView annotations]];
	
	PlaceMark* from = [[[PlaceMark alloc] initWithPlace:src] autorelease];
	PlaceMark* to = [[[PlaceMark alloc] initWithPlace:des] autorelease];
	
	[mapView addAnnotation:from];
	[mapView addAnnotation:to];
	
    /*
	routes = [[self calculateRoutesFrom:from.coordinate to:to.coordinate] retain];
    if (routes == nil) {
        <#statements#>
    }
    
    
    if (routes) {
        [self updateRouteView];
        [self centerMap];
    }*/
    
    [self calculateRoutesFrom:from.coordinate to:to.coordinate];
}

- (NSMutableArray *)decodePolyLine: (NSMutableString *)encoded {
	[encoded replaceOccurrencesOfString:@"\\\\" withString:@"\\"
								options:NSLiteralSearch
								  range:NSMakeRange(0, [encoded length])];
	NSInteger len = [encoded length];
	NSInteger index = 0;
	NSMutableArray *array = [[[NSMutableArray alloc] init] autorelease];
	NSInteger lat=0;
	NSInteger lng=0;
	while (index < len) {
		NSInteger b;
		NSInteger shift = 0;
		NSInteger result = 0;
		do {
			b = [encoded characterAtIndex:index++] - 63;
			result |= (b & 0x1f) << shift;
			shift += 5;
		} while (b >= 0x20);
		NSInteger dlat = ((result & 1) ? ~(result >> 1) : (result >> 1));
		lat += dlat;
		shift = 0;
		result = 0;
		do {
			b = [encoded characterAtIndex:index++] - 63;
			result |= (b & 0x1f) << shift;
			shift += 5;
		} while (b >= 0x20);
		NSInteger dlng = ((result & 1) ? ~(result >> 1) : (result >> 1));
		lng += dlng;
		NSNumber *latitude = [[[NSNumber alloc] initWithFloat:lat * 1e-5] autorelease];
		NSNumber *longitude = [[[NSNumber alloc] initWithFloat:lng * 1e-5] autorelease];
		printf("[%f,", [latitude doubleValue]);
		printf("%f]", [longitude doubleValue]);
		CLLocation *loc = [[[CLLocation alloc] initWithLatitude:[latitude floatValue] longitude:[longitude floatValue]] autorelease];
		[array addObject:loc];
	}
	
	return array;
}

- (void)calculateRoutesFrom:(CLLocationCoordinate2D)f to: (CLLocationCoordinate2D)t {
	NSString* saddr = [NSString stringWithFormat:@"%f,%f", f.latitude, f.longitude];
	NSString* daddr = [NSString stringWithFormat:@"%f,%f", t.latitude, t.longitude];
	
	NSString* apiUrlStr = [NSString stringWithFormat:@"http://maps.google.com/maps?hl=zh&output=dragdir&saddr=%@&daddr=%@", saddr, daddr];
    
	NSURL* apiUrl = [NSURL URLWithString:apiUrlStr];
    
    ASIHTTPRequest *request = [ASIHTTPRequest requestWithURL:apiUrl];
    
    [request setFailedBlock:^{
        if ([delegate respondsToSelector:@selector(CUMapPathViewRequestFailed:withError:)]) {
            [delegate CUMapPathViewRequestFailed:self withError:request.error];
        }
    }];
    
    [request setCompletionBlock:^{
        NSString *apiResponse = [request responseString];
        
        NSRegularExpression *regex = [NSRegularExpression regularExpressionWithPattern:@"points:\\\"([^\\\"]*)\\\"" 
                                                                               options:0 
                                                                                 error:nil];
        NSTextCheckingResult *match = [regex firstMatchInString:apiResponse 
                                                        options:0 
                                                          range:NSMakeRange(0, [apiResponse length])];
        NSString *encodedPoints = [apiResponse substringWithRange:[match rangeAtIndex:1]];
        
        self.routeArray = [self decodePolyLine:[encodedPoints mutableCopy]];
        
        [self updateRouteView:self.routeArray];
        [self centerMap:self.routeArray];
    }];
    
    [request startAsynchronous];
}

- (void)centerMap:(NSArray *)routes {
	MKCoordinateRegion region;
    
	CLLocationDegrees maxLat = -90;
	CLLocationDegrees maxLon = -180;
	CLLocationDegrees minLat = 90;
	CLLocationDegrees minLon = 180;
	for(int idx = 0; idx < routes.count; idx++)
	{
		CLLocation* currentLocation = [routes objectAtIndex:idx];
		if(currentLocation.coordinate.latitude > maxLat)
			maxLat = currentLocation.coordinate.latitude;
		if(currentLocation.coordinate.latitude < minLat)
			minLat = currentLocation.coordinate.latitude;
		if(currentLocation.coordinate.longitude > maxLon)
			maxLon = currentLocation.coordinate.longitude;
		if(currentLocation.coordinate.longitude < minLon)
			minLon = currentLocation.coordinate.longitude;
	}
	region.center.latitude     = (maxLat + minLat) / 2;
	region.center.longitude    = (maxLon + minLon) / 2;
	region.span.latitudeDelta  = maxLat - minLat;
	region.span.longitudeDelta = maxLon - minLon;
	
	[mapView setRegion:region animated:YES];
}

- (void)updateRouteView:(NSArray *)routes {
	CGContextRef context = 	CGBitmapContextCreate(nil, 
												  routeView.frame.size.width, 
												  routeView.frame.size.height, 
												  8, 
												  4 * routeView.frame.size.width,
												  CGColorSpaceCreateDeviceRGB(),
												  kCGImageAlphaPremultipliedLast);
	
	CGContextSetStrokeColorWithColor(context, lineColor.CGColor);
	CGContextSetRGBFillColor(context, 0.0, 0.0, 1.0, 1.0);
	CGContextSetLineWidth(context, 3.0);
	
	for(int i = 0; i < routes.count; i++) {
		CLLocation* location = [routes objectAtIndex:i];
		CGPoint point = [mapView convertCoordinate:location.coordinate toPointToView:routeView];
		
		if(i == 0) {
			CGContextMoveToPoint(context, point.x, routeView.frame.size.height - point.y);
		} else {
			CGContextAddLineToPoint(context, point.x, routeView.frame.size.height - point.y);
		}
	}
	
	CGContextStrokePath(context);
	
	CGImageRef image = CGBitmapContextCreateImage(context);
	UIImage* img = [UIImage imageWithCGImage:image];
	
	routeView.image = img;
	CGContextRelease(context);
}

#pragma mark mapView delegate functions
- (void)mapView:(MKMapView *)mapView regionWillChangeAnimated:(BOOL)animated
{
	routeView.hidden = YES;
}

- (void)mapView:(MKMapView *)mapView regionDidChangeAnimated:(BOOL)animated
{
	[self updateRouteView:self.routeArray];
	routeView.hidden = NO;
	[routeView setNeedsDisplay];
}

@end
